@@ -81,8 +81,8 @@ from yd_commands.variables import (
     L_TASK_NAME,
     L_TASK_NUMBER,
     L_WR_NAME,
-    add_substitution_overwrite,
-    add_substitutions,
+    add_or_update_substitution,
+    add_substitutions_without_overwriting,
     load_json_file_with_variable_substitutions,
     load_jsonnet_file_with_variable_substitutions,
     load_toml_file_with_variable_substitutions,
@@ -117,6 +117,10 @@ YD_TAG = "YD_TAG"
 
 @main_wrapper
 def main():
+
+    if not 1 <= TASK_BATCH_SIZE <= 10000:
+        raise Exception("Task batch size must be between 1 and 10,000")
+
     if ARGS_PARSER.json_raw:
         submit_json_raw(ARGS_PARSER.json_raw)
         return
@@ -250,7 +254,7 @@ def submit_work_requirement(
         wr_data.get(NAME, ID if CONFIG_WR.wr_name is None else CONFIG_WR.wr_name)
     )
     # Lazy substitution of the Work Requirement name, now it's defined
-    add_substitutions(subs={L_WR_NAME: ID})
+    add_substitutions_without_overwriting(subs={L_WR_NAME: ID})
     # Re-process substitutions in the CONFIG_WR object
     CONFIG_WR = update_config_work_requirement_object(CONFIG_WR)
     # Re-process substitutions in the wr_data dictionary
@@ -274,16 +278,12 @@ def submit_work_requirement(
 
     # Create the Work Requirement
     priority = check_float_or_int(wr_data.get(PRIORITY, CONFIG_WR.priority))
-    fulfilOnSubmit = check_bool(
-        wr_data.get(FULFIL_ON_SUBMIT, CONFIG_WR.fulfil_on_submit)
-    )
     work_requirement = WorkRequirement(
         namespace=CONFIG_COMMON.namespace,
         name=ID,
         taskGroups=task_groups,
         tag=CONFIG_COMMON.name_tag,
         priority=priority,
-        fulfilOnSubmit=fulfilOnSubmit,
     )
     if not ARGS_PARSER.dry_run:
         work_requirement = CLIENT.work_client.add_work_requirement(work_requirement)
@@ -369,12 +369,12 @@ def create_task_group(
     )
 
     # Add lazy substitutions for use in any Task Group property
-    add_substitution_overwrite(L_TASK_COUNT, str(num_tasks))
-    add_substitution_overwrite(L_TASK_GROUP_NAME, task_group_name)
-    add_substitution_overwrite(
+    add_or_update_substitution(L_TASK_COUNT, str(num_tasks))
+    add_or_update_substitution(L_TASK_GROUP_NAME, task_group_name)
+    add_or_update_substitution(
         L_TASK_GROUP_NUMBER, formatted_number_str(tg_number, num_task_groups)
     )
-    add_substitution_overwrite(L_TASK_GROUP_COUNT, str(num_task_groups))
+    add_or_update_substitution(L_TASK_GROUP_COUNT, str(num_task_groups))
     process_variable_substitutions_insitu(task_group_data)
     # Create a copy of global CONFIG_WR and apply lazy substitutions
     config_wr = update_config_work_requirement_object(deepcopy(CONFIG_WR))
@@ -560,12 +560,12 @@ def add_tasks_to_task_group(
         )
 
     # Add lazy substitutions for use in any Task property
-    add_substitution_overwrite(L_TASK_COUNT, str(num_tasks))
-    add_substitution_overwrite(L_TASK_GROUP_NAME, task_group.name)
-    add_substitution_overwrite(
+    add_or_update_substitution(L_TASK_COUNT, str(num_tasks))
+    add_or_update_substitution(L_TASK_GROUP_NAME, task_group.name)
+    add_or_update_substitution(
         L_TASK_GROUP_NUMBER, formatted_number_str(tg_number, num_task_groups)
     )
-    add_substitution_overwrite(L_TASK_GROUP_COUNT, str(num_task_groups))
+    add_or_update_substitution(L_TASK_GROUP_COUNT, str(num_task_groups))
 
     # Iterate through batches
     for batch_number in range(num_task_batches):
@@ -594,8 +594,8 @@ def add_tasks_to_task_group(
                 )
             )
 
-            add_substitution_overwrite(L_TASK_NAME, str(task_name))
-            add_substitution_overwrite(
+            add_or_update_substitution(L_TASK_NAME, str(task_name))
+            add_or_update_substitution(
                 L_TASK_NUMBER, formatted_number_str(task_number, num_tasks)
             )
             process_variable_substitutions_insitu(task)
@@ -619,10 +619,20 @@ def add_tasks_to_task_group(
                 task.get(ENV, task_group_data.get(ENV, wr_data.get(ENV, config_wr.env)))
             )
 
+            add_yd_env_vars = check_bool(
+                task.get(
+                    ADD_YD_ENV_VARS,
+                    task_group_data.get(
+                        ADD_YD_ENV_VARS,
+                        wr_data.get(ADD_YD_ENV_VARS, config_wr.add_yd_env_vars),
+                    ),
+                )
+            )
+
             # Task timeout is automatically inherited from the Task Group level
             # unless overridden by the Task
             task_timeout_minutes = check_float_or_int(
-                task.get(TASK_LEVEL_TIMEOUT, None)
+                task.get(TASK_LEVEL_TIMEOUT, CONFIG_WR.task_level_timeout)
             )
             task_timeout = (
                 None
@@ -839,6 +849,7 @@ def add_tasks_to_task_group(
                     outputs=outputs,
                     task_timeout=task_timeout,
                     flatten_upload_paths=flatten_upload_paths,
+                    add_yd_env_vars=add_yd_env_vars,
                 )
             )
 
@@ -934,8 +945,9 @@ def follow_progress(work_requirement: WorkRequirement) -> None:
     Follow and report the progress of a Work Requirement.
     Replacement for the SDK version above.
     """
-    print_log("Following Work Requirement event stream")
-    follow_events(work_requirement.id, YDIDType.WORK_REQ)
+    if not ARGS_PARSER.dry_run:
+        print_log("Following Work Requirement event stream")
+        follow_events(work_requirement.id, YDIDType.WORK_REQ)
 
 
 def on_update(work_req: WorkRequirement):
@@ -1104,6 +1116,7 @@ def create_task(
     outputs: Optional[List[TaskOutput]],
     task_timeout: Optional[timedelta],
     flatten_upload_paths: bool = False,
+    add_yd_env_vars: bool = False,
 ) -> Task:
     """
     Create a Task object, handling special processing for specific Task Types.
@@ -1145,7 +1158,7 @@ def create_task(
         flatten_input_paths = FlattenPath.FILE_NAME_ONLY
 
     # Add Task details to the environment as a convenience
-    if task_type != "docker":
+    if task_type != "docker" and add_yd_env_vars:
         env_copy[YD_TASK_NAME] = task_name
         env_copy[YD_TASK_NUMBER] = str(task_number)
         env_copy[YD_TASK_GROUP_NAME] = tg_name
@@ -1213,20 +1226,34 @@ def create_task(
                 ),
             )
         )
-        # Add entity names to the container env. for convenience
-        docker_env_list = ["--env", f"{YD_TASK_NAME}={task_name}"]
-        docker_env_list += ["--env", f"{YD_TASK_NUMBER}={task_number}"]
-        docker_env_list += ["--env", f"{YD_TASK_GROUP_NAME}={tg_name}"]
-        docker_env_list += ["--env", f"{YD_TASK_GROUP_NUMBER}={tg_number}"]
-        docker_env_list += ["--env", f"{YD_WORK_REQUIREMENT_NAME}={ID}"]
-        docker_env_list += ["--env", f"{YD_NAMESPACE}={CONFIG_COMMON.namespace}"]
-        docker_env_list += ["--env", f"{YD_TAG}={CONFIG_COMMON.name_tag}"]
+        docker_env_list = []
+        if add_yd_env_vars:
+            # Add entity names to the container env. for convenience
+            docker_env_list += ["--env", f"{YD_TASK_NAME}={task_name}"]
+            docker_env_list += ["--env", f"{YD_TASK_NUMBER}={task_number}"]
+            docker_env_list += ["--env", f"{YD_TASK_GROUP_NAME}={tg_name}"]
+            docker_env_list += ["--env", f"{YD_TASK_GROUP_NUMBER}={tg_number}"]
+            docker_env_list += ["--env", f"{YD_WORK_REQUIREMENT_NAME}={ID}"]
+            docker_env_list += ["--env", f"{YD_NAMESPACE}={CONFIG_COMMON.namespace}"]
+            docker_env_list += ["--env", f"{YD_TAG}={CONFIG_COMMON.name_tag}"]
 
         if docker_env is not None:
             for key, value in docker_env.items():
                 docker_env_list += ["--env", f"{key}={value}"]
 
-        args = docker_env_list + [executable] + args
+        # Check for any Docker options
+        docker_options = check_list(
+            task_data.get(
+                DOCKER_OPTIONS,
+                task_group_data.get(
+                    DOCKER_OPTIONS,
+                    wr_data.get(DOCKER_OPTIONS, config_wr.docker_options),
+                ),
+            )
+        )
+        docker_options = [] if docker_options is None else docker_options
+
+        args = docker_env_list + docker_options + [executable] + args
 
         # Set up the environment used by the script to run Docker
         # Add the username and password, if present
@@ -1282,7 +1309,7 @@ def submit_json_raw(wr_file: str):
     # Lazy substitution of Work Requirement name
     wr_data["name"] = format_yd_name(wr_data["name"])
     wr_name = wr_data["name"]
-    add_substitutions(subs={L_WR_NAME: wr_name})
+    add_substitutions_without_overwriting(subs={L_WR_NAME: wr_name})
     process_variable_substitutions_insitu(wr_data)
 
     if ARGS_PARSER.dry_run:
